#!/usr/bin/perl -w
#
#  Dump FLAK Battle files (FLAK.HST / FLAKx.DAT) to standard output
#
use strict;

sub print_it;

foreach my $filename (@ARGV) {
    print "=== $filename ===\n";

    open FILE, "< $filename" or die;
    binmode FILE;

    # header
    my @header = unpack "A8vvvvA18V", get_bytes(8+4*2+18+4);
    if ($header[0] ne "FLAKVCR\032") { die "not a FLAK file" }
    print "+ Header:\n";
    print_it ['File format version', 'Player', 'Turn', 'Number of battles',
              'Timestamp', 'Reserved'], @header[1 .. $#header];

    # battle
    foreach (1 .. $header[4]) {
        print "+ Battle #$_:\n";
        my $data = get_bytes (4);
        if (!defined $data) { die "unable to read" }
        my $size = unpack "V", $data;
        if ($size <= 4) { die "bad size $size" }
        $data .= get_bytes($size-4);
        my @list = unpack "VvvVVVVVVVVVVVV", $data;
        print_it ['Size','X','Y','Seed','Time','Ambient flags',
                  'Number of fleets', '  Fleet entry size', '  Fleet pointer',
                  'Number of ships',  '  Ship entry size',  '  Ship pointer',
                  'Attack list entries', '  Att entry size', '  Att pointer'],
                    @list;

        # fleets
        foreach (0 .. $list[6]-1) {
            print "  - Fleet #$_:\n";
            print_it ["  Player", "  First ship", "  Number of ships",
                      "  Speed", "  Att list pointer", "  Att list size",
                      "  X", "  Y"],
                        unpack "vvvvVVVV", substr($data, $_*$list[7] + $list[8], $list[7]);
        }

        # ships
        foreach (0 .. $list[9]-1) {
            print "  - Ship #$_:\n";
            print_it ["  Name", "  Damage", "  Crew", "  Id", "  Player",
                      "  Hull", "  Level", "  Beam count", "  Beam type",
                      "  Torp Launchers", "  Torp count", "  Torp type",
                      "  Bay count", "  Fighter count", "  Mass",
                      "  Shields", "  Max ftrs launched", "  Rating",
                      "  Compensation", "  Flags", "  Result"],
                        unpack "A20vvvvvvvvvvvvvvvvVvvv", substr($data, $_*$list[10] + $list[11], $list[10]);
        }

        # ships
        print "  - Attack list:\n";
        foreach (0 .. $list[12]-1) {
            my @entry = unpack "vv", substr($data, $_*$list[13] + $list[14], $list[13]);
            if (@entry == 2) {
                printf "    [%04d] attack ship %d with rating %d\n", $_, @entry;
            }
        }
    }

    close FILE;
}
exit 0;

sub print_it {
    my $listptr = shift;
    foreach (@$listptr) {
        my $elem = @_ ? shift @_ : '<undef>';
        if ($elem !~ /\D/ && $elem > 3000000000) { $elem -= 2**32 }
        printf "  %-20s %s\n", "$_:", $elem;
    }
}

sub get_bytes {
    my $count = shift;
    my $buf;
    read FILE, $buf, $count;
    $buf;
}
